/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software and design ideas developed by the Java
 *    Apache Project (http://java.apache.org/)."
 *
 * 4. The names "Cocoon", "Cocoon Servlet" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Cocoon"
 *    nor may "Cocoon" and "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software and design ideas developed by the Java
 *    Apache Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 */

package org.apache.cocoon;

import java.util.*;
import org.apache.cocoon.framework.*;

/**
 * This inner class is used to store the mapping between browser names
 * and those signature fragments used for mapping.
 *
 * @author <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @version $Revision: 1.7 $ $Date: 1999/08/14 22:15:27 $
 */

public class Browsers extends Vector implements Configurable, Status {

    /**
     * Inner class that contains single browser information.
     */
    public class Browser {

        public String name;
        public String signature;

        public Browser(String name, String signature) {
            this.name = name;
            this.signature = signature;
        }

        public String toString() {
            return "( " + name + ", " + signature + " )";
        }
    }

    /**
     * Initialize the map with the mapping rules.
     */
    public void init(Configurations conf) throws InitializationException {
        for (int i = 0; ; i++) {
            String filter = (String) conf.get(Integer.toString(i));
            if (filter == null) break;
            int index = filter.indexOf('=');
            this.addElement(new Browser(filter.substring(0, index), filter.substring(index + 1)));
        }
    }

    /**
     * This method maps the "user-Agent" request parameter to a browser name
     * following the search rules defined in the servlet properties.
     */
    public String map(String agent) {
        try {
            for (int i = 0;; i++) {
                Browser b = (Browser) this.elementAt(i);
                int index = agent.indexOf(b.signature);
                if (index > -1) return b.name;
            }
        } catch (Exception e) {
            return "unknown";
        }
    }

    public String getStatus() {
        StringBuffer buffer = new StringBuffer();

        for (int i = 0; i < this.size(); i++) {
            Browser b = (Browser) this.elementAt(i);
            buffer.append("<li>");
            buffer.append(b.toString());
            buffer.append("</li>");
        }

        return buffer.toString();
    }
}